Package: phenopath
Type: Package
Title: Genomic trajectories with heterogeneous genetic and
        environmental backgrounds
Version: 1.34.0
Date: 2017-10-24
Author: Kieran Campbell
Maintainer: Kieran Campbell <kieranrcampbell@gmail.com>
Description: PhenoPath infers genomic trajectories (pseudotimes) in the presence of
  heterogeneous genetic and environmental backgrounds and tests for interactions 
  between them.
License: Apache License (== 2.0)
Imports: Rcpp (>= 0.12.8), SummarizedExperiment, methods, stats, dplyr,
        tibble, ggplot2, tidyr
Suggests: knitr, rmarkdown, forcats, testthat, BiocStyle,
        SingleCellExperiment
biocViews: ImmunoOncology, RNASeq, GeneExpression, Bayesian,
        SingleCell, PrincipalComponent
LinkingTo: Rcpp
RoxygenNote: 6.0.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/phenopath
git_branch: RELEASE_3_22
git_last_commit: f09cdb5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 05:42:12 UTC; biocbuild
