# Generated by roxygen2: do not edit by hand

S3method(print,phenopath_fit)
export(clvm)
export(interaction_effects)
export(interaction_sds)
export(interactions)
export(phenopath)
export(plot_elbo)
export(significant_interactions)
export(simulate_phenopath)
export(trajectory)
import(Rcpp)
import(ggplot2)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,mutate)
importFrom(methods,is)
importFrom(stats,coef)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,prcomp)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(tibble,as_data_frame)
importFrom(tidyr,gather)
useDynLib(phenopath)
